(in-package :araneida)

(defvar *sockets-for-fds* (make-hash-table :test 'eql))

(defun listen-to-inet-port (&key (port 0) (kind :stream) (reuse nil))
  "Create, bind and listen to an inet socket on *:PORT.
setsockopt SO_REUSEADDR if :reuse is not nil"
  (let ((socket (make-instance 'sockets:inet-socket
			       :type :stream
			       :protocol :tcp)))
    (if reuse
        (setf (sockets:sockopt-reuse-address socket) t))
    (sockets:socket-bind socket (sockets:make-inet-address "0.0.0.0") port)
    (sockets:socket-listen socket 15)
    socket))

(defun close-socket (fd)
  (sockets:socket-close (gethash fd *sockets-for-fds*))
  (remhash fd *sockets-for-fds*))

;; XXX do we have the right stream element type
(defun make-stream-from-fd (fd &key (input t) (output t) (buffering :none))
  (sockets:socket-make-stream
   (gethash fd *sockets-for-fds*)
   :input input :output output :buffering buffering))


(defun forcibly-close-stream (s)
  (let ((fd (sb-sys:fd-stream-fd s)))
    (multiple-value-bind (r e) (ignore-errors (close s) t)
      (unless r
	(format t "Unable to close fd ~A: ~A, trying harder ~%" fd e)
	(multiple-value-bind (r e) (ignore-errors (close s :abort t) t)
	  (unless r
	    (format t "still unable to close ~A: ~A, try harder ~%" fd e)
	    (multiple-value-bind (r e)
		(ignore-errors (sb-unix:unix-close fd) t)
	      (unless r
		(format t "Even unix-close failed on ~A:~A, giving up~%"
			fd e)))))))))
    
